/*
**  PlaylistManager.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "PlaylistManager.h"
#include <MusicBox/AudioList.h>
#include <MusicBox/ModuleLoader.h>
#include <ShengGuang/SGSystem.h>
#include <ShengGuang/SongInfo.h>
#include "MusicBoxWindowController.h"
#include <Foundation/Foundation.h>

static PlaylistManager *sharedInstance = nil;

@implementation PlaylistManager

- (void) setMainWindowController: (MusicBoxWindowController *) controller
{
  ASSIGN(mainWindowController, controller);
}

- (MusicBoxWindowController *) mainWindowController
{
  return mainWindowController;
}

- (void) syncPlaylist
{
  NSArray *cacheArray;
  SongInfo *info;
  int i, count, index = -1;

  selectedPlaylistForPlay = selectedPlaylist;
  info = [mainWindowController selectedSongInfo];
  cacheArray = [NSArray arrayWithArray: [self soundListForPlay]];
  count = [cacheArray count];
  /* Due to the nature that soundList may change from time to time.
   * Have to use the brutal way to find out the right index.
   * It works if and only if no duplicated description
   */
  for (i = 0; i < count; i++)
    {
      if ([[[cacheArray objectAtIndex: i] description] isEqualToString: [info description]])
        {
          index = i;
          break;
        }
    }
  if (index > -1)
    [self selectSongForPlay: index];
  else
    NSLog(@"PlaylistManager: FIXME: no selectedSongForPlay");
}

+ (id) sharedManager
{
  if (!sharedInstance)
    {
      sharedInstance = [[PlaylistManager alloc] initFromFile];
    }
  return sharedInstance;
}

- (SongInfo *) selectedSongInfoForPlay
{
  return [self songInfoForPlay: [self indexOfSelectedSongForPlay]];
}

- (SongInfo *) songInfoForPlay: (int) index
{
  if ((index < [[self soundListForPlay] count]) && (index > -1))
    return [[self soundListForPlay] objectAtIndex: index];
  else 
    return nil;
}

- (void) selectSongForPlay: (int) index
{
  selectedSongForPlay = index;
}

- (int) indexOfSelectedSongForPlay
{
  return selectedSongForPlay;
}

- (void) selectPlaylistForPlay: (int) index
{
  selectedPlaylistForPlay = index;
}

- (AudioList *) selectedPlaylistForPlay
{
  return [sourceList objectAtIndex: selectedPlaylistForPlay];
}

- (NSArray *) soundListForPlay
{
  return [[self selectedPlaylistForPlay] soundList];
}

- (void) removeSelectedPlaylist
{
  if ((selectedPlaylist > -1) && ([self count] > 0))
    [self removePlaylist: selectedPlaylist];
}

- (AudioList *) selectedPlaylist
{
  if ((selectedPlaylist > -1) && ([self count] > 0))
    return [self playlistAt: selectedPlaylist];
  else
    return nil;
}

- (int) indexOfSelectedPlaylist
{
  return selectedPlaylist;
}

- (void) selectPlaylist: (int) index
{
  selectedPlaylist = index;
}

- (id) init
{
  self = [super init];
  
  sourceList = [NSMutableArray new];
  selectedPlaylistForPlay = -1;
  selectedPlaylist = -1;
  selectedSongForPlay = -1;

  return self;
}

- (id) initFromFile
{
  NSMutableArray *mainLibrary;
  ModuleLoader *moduleLoader;
  Class moduleClass;
  AudioList * media;
  NSDictionary *property = nil;
  NSArray *list = nil;
  NSString *name = nil;
  NSString *path;
  int i;

  self = [self init];

  /* Check the existing of playlist file */
  path = [[NSUserDefaults standardUserDefaults] objectForKey: @"PlaylistFile"];
  if (path == nil)
    {
      /* Use the default value */
      [[NSUserDefaults standardUserDefaults] setObject: @"~/.MusicBox"
                                                forKey: @"PlaylistFile"];
      /* Then the mainLibrary must be nil because it doesn't existing */
      mainLibrary = [NSMutableArray new];
    }
  else
    {
      /* Set up source list, must have "Library" as all music files */
      mainLibrary = [[NSMutableArray alloc] initWithContentsOfFile: [path stringByExpandingTildeInPath]];
    }

// Main Library, which is unique

  moduleLoader = [ModuleLoader defaultLoader];
  moduleClass = [moduleLoader classNameOf: @"AudioLibrary.audio"];
  media = [[moduleClass alloc] initWithOwner: self];

  for(i = 0; i < [mainLibrary count]; i++)
    {
      if ([[[mainLibrary objectAtIndex: i] objectForKey: @"class"]
                         isEqualToString: @"AudioLibrary"]) 
        {
          list = [[mainLibrary objectAtIndex: i] objectForKey: @"list"];
          property = [[mainLibrary objectAtIndex: i] objectForKey: @"property"];
          name = [[mainLibrary objectAtIndex: i] objectForKey: @"name"];
          [mainLibrary removeObjectAtIndex: i];
          break;
        }
    }

  if ([property objectForKey: @"path"] == nil)
    {
      int result;

      result = NSRunAlertPanel(_(@"No music library. Want to create ?"),
       _(@"MusicBox will use ~/MusicBox to store your music collection.\nYou can always change it to an existing directory later in the property of Library."),
                          _(@"OK"), _(@"Cancel"), nil);
      if (result == NSAlertDefaultReturn)
        {
          if ([[NSFileManager defaultManager] createDirectoryAtPath:
                     [@"~/MusicBox" stringByStandardizingPath]
                                          attributes: nil] == NO)
            {
                NSRunAlertPanel(_(@"Can't create the music library"),
           _(@"Warning: You have to specify an existing directory in Library."),
                          _(@"OK"), nil, nil);
            }
        }
      property = [NSMutableDictionary dictionaryWithObject: @"~/MusicBox" 
                                                  forKey: @"path"];
    }

  if (name == nil)
    [media setName: _(@"Library")];
  else 
    [media setName: name];

  [media setProperty: property];
  [media loadData: list];

  NSLog(@"main %d", [media count]);
  [self addPlaylist: media];
  RELEASE(media);

// Load Audio CD
  if ([[SGSystem sharedSystem] hasCDROM])
    {
      NSLog(@"Load Audio CD");
      moduleClass = [moduleLoader classNameOf: @"AudioCD.audio"];
      if (moduleClass)
        {
          media = [[moduleClass alloc] initWithOwner: self];
          [media setName: _(@"Audio CD")];

          [self addPlaylist: media];
          RELEASE(media);
        }
    }
// Load other playlist

  for(i = 0; i < [mainLibrary count]; i++) {
    NSString *audioClass = [[mainLibrary objectAtIndex: i] objectForKey: @"class"];
    if ([audioClass isEqualToString: @"AudioList"])
      {
        media = [[AudioList alloc] initWithOwner: self];
      }
    else // Load from audio module
      {
        moduleClass = [moduleLoader classNameOf: [audioClass stringByAppendingPathExtension: @"audio"]];
        media = [[moduleClass alloc] initWithOwner: self];
      }
    [media loadData: [[mainLibrary objectAtIndex: i] objectForKey: @"list"]];
    [media setProperty: [[mainLibrary objectAtIndex: i] objectForKey: @"property"]];
    [media setName: [[mainLibrary objectAtIndex: i] objectForKey: @"name"]];
    [self addPlaylist: media];
    RELEASE(media);
  }

  NSLog(@"Finish init PlaylistManager");
  RELEASE(mainLibrary);
  return self;
}

- (void) writeToFile
{
  int i;
  id object, property;
  NSMutableDictionary *dictionary;
  NSMutableArray *array;
       
// Write play list into file
  array = [NSMutableArray new];
  for(i = 0; i < [sourceList count]; i++) {
    object = [[sourceList objectAtIndex: i] dataForSave];
    property = [[sourceList objectAtIndex: i] property];
    if((object != nil) || (property)){ 
      dictionary = [NSMutableDictionary new];
      [dictionary setObject: [(AudioList *)[sourceList objectAtIndex: i] name]
                     forKey: @"name"];
      [dictionary setObject: [(AudioList *)[sourceList objectAtIndex: i] class]   
                     forKey: @"class"];
      if (property != nil)
        {
          [dictionary setObject: property
                         forKey: @"property"];
        }
      if (object != nil)
        {
          [dictionary setObject: object
                         forKey: @"list"];
        }

      [array addObject: dictionary];
      RELEASE(dictionary);
    }
  }
  [array writeToFile: [[[NSUserDefaults standardUserDefaults]
                                        stringForKey: @"PlaylistFile"]
                              stringByExpandingTildeInPath]
          atomically: YES];
  
  RELEASE(array);
}

- (NSArray *) allNamesOfPlaylists
{
  int i;
  unsigned count;
  NSMutableArray *array = [NSMutableArray new];
  count = [self count];
  for(i = 0; i < count; i++)
    {
      [array addObject: [[self playlistAt: i] name]];
    }
  return AUTORELEASE(array);
}

- (NSArray *) allPlaylist
{
  return sourceList;
}

- (unsigned) count
{
  return [sourceList count];
}

- (void) addPlaylist: (AudioList *) playlist
{
  [sourceList addObject: playlist];
  selectedPlaylist = [sourceList count] - 1;
}

- (void) movePlaylist: (int) fromIndex to: (int) toIndex
{
  id object;

  if (fromIndex == toIndex)
    return;

  object = [sourceList objectAtIndex: fromIndex];
  RETAIN(object);

  [sourceList removeObjectAtIndex: fromIndex];
  if (toIndex > [sourceList count] - 1)
    {
      [sourceList addObject: object];
    } 
  else if (fromIndex > toIndex)
    {
      [sourceList insertObject: object atIndex: toIndex];
    }
  else if (fromIndex < toIndex)
    {
      [sourceList insertObject: object atIndex: toIndex-1];
    }
  RELEASE(object);
}

- (void) removePlaylist: (int) index
{
  /* Prevent removing the main library */
  if ((index < 1) || (index >= [sourceList count]))
    return;
  [sourceList removeObjectAtIndex: index];
  if ([sourceList count] < 1)
    selectedPlaylist = -1;
  else if (selectedPlaylist >= [sourceList count])
    selectedPlaylist = [sourceList count] -1;
  else
    selectedPlaylist = index;
}

- (AudioList *) playlistAt: (int) index
{
  return [sourceList objectAtIndex: index];
}

- (AudioList *) mainLibrary
{
  int i;
  unsigned count = [self count];
  for (i = 0; i < count; i ++)
    {
      NSString *class = [[[self playlistAt: i] class] description];
      if ([class isEqualToString: @"AudioLibrary"])
        return [self playlistAt: i];
    }
  return nil;
}

- (void) dealloc
{
  RELEASE(sourceList);
  [super dealloc];
}

@end
